<?php

namespace App\Service;

use App\Exception;

class ProductService {
    
    private $util;
    private $factory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ProductFactory $factory) {
        $this->util    = $util;
        $this->factory = $factory;
    }
    
    public function processProduct ($generalParams, $product, $order, &$nrSeqProdCom, &$ITCOMANDAVEN_SEQ) {
        
        // Valida se é Combo
        if (!empty($product["properties"]) && count($product["properties"]) > 0) {
            //Combo com código externo
            if (!empty($product["customCode"])) {
                $processedProduct = $this->processProductAsComboWithExtCode($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);
            } else { //Combo sem código externo
                $processedProduct = $this->processProductAsComboWithoutExtCode($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);    
            }
        } else {
            // Produto Simples
            $processedProduct = $this->processProductAsSimple($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);
        }
        
        return $processedProduct;
    }
    
    private function processProductAsSimple ($generalParams, $product, $order, &$ITCOMANDAVEN_SEQ) {
        $simpleItens = [];

        if (!isset($product["customCode"]) || empty($product["customCode"]))
            throw new Exception\ParamsException("
                O produto {$product['name']} não possui código externo e ele é obrigatório.1<br>
                Verifique no Odhen Retail o código externo deste produto e faça a correção junto ao Delivery Direto.<br>
                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
        
        $productDetails = $this->factory->getProduct($product["customCode"]);
        
        if (empty($productDetails))
            throw new Exception\ParamsException("
                Não existe um produto atrelado ao código externo informado.<br>
                Produto: {$product['name']}.<br>
                Código externo: {$product['customCode']}.<br>
                Verifique no Odhen Retail o código externo deste produto e faça a correção junto ao Delivery Direto.2<br>
                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
        
        $productDetails = array_merge($productDetails, $product);
        
        $productDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
        $ITCOMANDAVEN_SEQ++;
        
        $productDetails["TXPRODCOMVEN"] = (isset($product["comments"]) && !empty($product["comments"])) ? (strtoupper($this->util->removeAcentos($product["comments"]))) . "; " : "";
        
        $itemReturn = array(
            "CDFILIAL"       => $generalParams["CDFILIAL"],
            "NRVENDAREST"    => $order["NRVENDAREST"],
            "NRCOMANDA"      => $order["NRCOMANDA"],
            "NRPRODCOMVEN"   => $productDetails["NRPRODCOMVEN"],
            "CDPRODUTO"      => $productDetails["CDPRODUTO"],
            "NMPRODUTO"      => $productDetails["name"],
            "QTPRODCOMVEN"   => $product["amount"],
            "VRPRECCOMVEN"   => bcdiv($productDetails["price"]["value"], "100", 2),
            "TXPRODCOMVEN"   => $productDetails["TXPRODCOMVEN"],
            "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
            "VRDESCCOMVEN"   => isset($productDetails['VRDESCCOMVEN']) ? $productDetails['VRDESCCOMVEN'] : 0,
            "NRLUGARMESA"    => "01",
            "IDPRODIMPFIS"   => "N",
            "CDLOJA"         => $generalParams["CDLOJA"],
            "VRACRCOMVEN"    => 0,
            "NRSEQPRODCOM"   => null,
            "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
            "VRPRECCLCOMVEN" => 0,
            "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
            "CDPRODPROMOCAO" => null,
            "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
            "NRINSCRCONS"    => $generalParams["CPF"],
            "DSOBSPEDDIGCMD" => null
        );
    
        array_push($simpleItens, $itemReturn);
        
        return $simpleItens;
    }
    
    private function processProductAsComboWithExtCode ($generalParams, $combo, $order, &$ITCOMANDAVEN_SEQ, &$comboItens = array(), $CDPRODPROMO = null) {
        $comboItens = array();

        for ($i = 0 ; $i < $combo["amount"] ; $i++) {
            if(!isset($combo["customCode"]) || empty($combo["customCode"]))
                throw new Exception\ParamsException("<p align='justify'>
                    O produto {$combo['name']} foi cadastrado com preço e sem código externo. A obrigatoriedade do preço e do código externo dependem das regras abaixo:<br>
                    Caso o produto tenha preço, será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste produto, no formato 'x.xx.xx.xxx.xx'.<br>
                    Caso o produto seja um agrupador, ele não deve ter preço, nem código externo.<br>
                    O código externo (caso exista) deve ser igual ao cadastrado previamente no Odhen Retail.<br>
                    Para mais informações, entre em contato com a equipe de suporte da Teknisa.</p>");

            $comboDetails = $this->factory->getProduct($combo["customCode"]);

            if (empty($comboDetails))
                throw new Exception\ParamsException("
                    Não existe um produto atrelado ao código externo informado.<br>
                    Produto: {$combo['name']}.<br>
                    Código externo: {$combo['customCode']}.<br>
                    Verifique no Odhen Retail o código externo deste produto e faça a correção no portal Pede Pronto.");

            $comboDetails = array_merge($comboDetails, $combo);
            
            $comboDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            $productNrSeqProdCom = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 3, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;
            
            $comboDetails["TXPRODCOMVEN"] = (isset($combo["comments"]) && !empty($combo["comments"])) ? (strtoupper($this->util->removeAcentos($combo["comments"]))) . "; " : "";

            $itemReturn = array(
                "CDFILIAL"         => $generalParams["CDFILIAL"],
                "NRVENDAREST"      => $order["NRVENDAREST"],
                "NRCOMANDA"        => $order["NRCOMANDA"],
                "NRPRODCOMVEN"     => $comboDetails["NRPRODCOMVEN"],
                "CDPRODUTO"        => $comboDetails["CDPRODUTO"],
                "NMPRODUTO"        => $comboDetails["name"],
                "QTPRODCOMVEN"     => 1,
                "VRPRECCOMVEN"     => bcdiv($comboDetails["price"]["value"], "100", 2),
                "TXPRODCOMVEN"     => $comboDetails["TXPRODCOMVEN"],
                "IDSTPRCOMVEN"     => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                "VRDESCCOMVEN"     => isset($comboDetails['VRDESCCOMVEN']) ? $comboDetails['VRDESCCOMVEN'] : 0,
                "NRLUGARMESA"      => "01",
                "IDPRODIMPFIS"     => "N",
                "CDLOJA"           => $generalParams["CDLOJA"],
                "VRACRCOMVEN"      => 0,
                "NRSEQPRODCOM"     => (!empty($comboDetails['propertiesPrice']) && $comboDetails['propertiesPrice'] >= 0.01) ? $productNrSeqProdCom : null,
                "NRSEQPRODCUP"     => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"   => 0,
                "CDCAIXACOLETOR"   => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO"   => $comboDetails["CDPRODUTO"],
                "CDVENDEDOR"       => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"      => $generalParams["CPF"],
                "DSOBSPEDDIGCMD"   => null,
                "ITEM_OBSERVATION" => []
            );

            $CDPRODPROMO = empty($CDPRODPROMO) ? $comboDetails["CDPRODUTO"] : $CDPRODPROMO;

            array_push($comboItens, $itemReturn);

            foreach ($combo["properties"] as $comboItemAgrupador) {
                foreach ($comboItemAgrupador["options"] as $comboItem){
                    
                    //========================== Observações Sem Preço ==========================
                    if(!empty($comboItem['name']) && (empty($comboItem['price']['value']) || $comboItem['price']['value'] == "0.00")) {
                        // Observações tipo O e R
                        if(!isset($comboItem['customCode']) || empty($comboItem['customCode']))
                            throw new Exception\ParamsException("
                                O subitem {$comboItem['name']}, do item {$combo['name']}, foi cadastrado sem preço e sem código externo. Apesar do preço não ser uma exigência (ver abaixo), o código externo é obrigatório para todos os subitems.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'x.xx.xx.xxx.xx' ou no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                O código externo deve ser igual ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
    

                        if(!strpos($comboItem['customCode'], "|"))
                            throw new Exception\ParamsException("
                                O subitem {$comboItem['name']}, do item {$combo['name']}, foi cadastrado sem preço e não existe um subitem cadastrado no Retail com o código externo informado: {$comboItem['customCode']}.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código externo deste subitem. O formato correto é Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
    
                        $validateObs = $this->factory->getObservation($comboItem['customCode']);
                        
                        if($validateObs['IDCONTROLAOBS'] === "A"){
                            throw new Exception\ParamsException("
                                O subitem {$comboItem['name']}, do item {$combo['name']}, foi cadastrado como um acréscimo utilizando o código no padrão XX|YY, mas está sem preço.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código do item.<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
                        }

                        if(!count($validateObs))
                            throw new Exception\ParamsException("
                                Não existe uma observação atrelada ao código externo informado.<br>
                                Item do pedido: {$comboItem['name']}.<br>
                                Código externo: {$comboItem['customCode']}.<br>
                                Verifique no Odhen Retail o código externo desta observação e faça a correção junto ao Delivery Direto.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
                        
                        array_push($comboItens[0]['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],
                            'NRVENDAREST'     => $order["NRVENDAREST"],
                            'NRCOMANDA'       => $order["NRCOMANDA"],
                            'NRPRODCOMVEN'    => $comboDetails["NRPRODCOMVEN"],
                            'CDGRPOCOR'       => $validateObs['CDGRPOCOR'],
                            'CDOCORR'         => $validateObs['CDOCORR'],
                            'NRPRODCOMVENOBS' => null,
                            'QTPRODCOMVENOBS' => $comboItem['amount']
                        ]);
    
                        $this->processObservationWithoutPrice($comboDetails["NRPRODCOMVEN"], $validateObs['DSOCORR'], $comboItem['amount'], $comboItens);
                        continue;                    
                    }
                    //======================= Fim de Observações Sem Preço ======================
                    
                    // QUANTIDADE DE ACRÉSCIMOS UM PARA CADA ACRÉSCIMO. SE TIVER 5 + BACON, SERÁ GRAVADO 1x BACON, 5 vezes
                    
                    if (strlen($comboItem["customCode"]) === 5 && substr($comboItem["customCode"], 2, 1) === '|' && $comboItem['price']['value'] != "0.00"){
                        $itemDetails = $this->factory->getObservation($comboItem["customCode"]);
                    } else {
                        $itemDetails = $this->factory->getProduct($comboItem["customCode"]);
                    }

                    if (empty($itemDetails)){
                        if(strpos($comboItem['customCode'], "|")){
                            throw new Exception\ParamsException("
                                O subitem '{$comboItem['name']}' possui preço ({$comboItem['price']['value']}) e o código externo ({$comboItem['customCode']}) está no formato Código Grupo de Observação|Código de Observação (XX|YY).<br>
                                Caso o subitem seja uma observação ou remoção (por exemplo, 'Sem salada' ou 'Ao ponto'), você deve remover o preço deste subitem.<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), você deve corrigir o código externo deste subitem.<br>
                                O código externo e o valor do subitem devem ser iguais ao cadastrado previamente no Odhen Retail.<br>
                                Verifique no Retail o código externo e o valor desta observação e faça a correção junto ao Delivery Direto.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
                        }
                        if(empty($comboItem['customCode'])){
                            throw new Exception\ParamsException("
                            Não existe observação ou acréscimo atrelado ao código externo informado.<br>
                            Observação ou acréscimo: {$comboItem['name']}.<br>
                            Produto pai: {$combo['name']}.<br>
                            Verifique no Retail o código externo desta observação e faça a correção junto ao Delivery Direto.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");    
                        }
                        throw new Exception\ParamsException("
                            Não existe um produto atrelado ao código externo informado.3<br>
                            Produto: {$comboItem['name']}.<br>
                            Código externo: {$comboItem['customCode']}.<br>
                            Verifique no Retail o código externo e o valor desta observação e faça a correção junto ao Delivery Direto.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
                    }

                    if(isset($itemDetails['IDCONTROLAOBS']) && $itemDetails['IDCONTROLAOBS'] != "A" && strpos($comboItem['customCode'], "|")){
                            throw new Exception\ParamsException("
                                O subitem {$comboItem['name']}, do item {$combo['name']} foi cadastrado como um acréscimo utilizando o código no padrão XX|YY, mas o produto encontrado ({$itemDetails['DSOCORR']}) foi uma observação.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código do item.<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
                    }
                        
                    $itemDetails = array_merge($itemDetails, $comboItem);
                    
                    for($j = 0; $j < $comboItem["amount"]; $j++){
                        $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                        $ITCOMANDAVEN_SEQ++;
    
                        $itemDetails["TXPRODCOMVEN"] = (isset($comboItem["comments"]) && !empty($comboItem["comments"])) ? (strtoupper($this->util->removeAcentos($comboItem["comments"]))) . "; " : "";
                        
                        $comboItemPrice = bcdiv($itemDetails["price"]["value"], $comboItem["amount"], 2);
                        
                        $itemReturn = array(
                            "CDFILIAL"       => $generalParams["CDFILIAL"],
                            "NRVENDAREST"    => $order["NRVENDAREST"],
                            "NRCOMANDA"      => $order["NRCOMANDA"],
                            "NRPRODCOMVEN"   => $itemDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"      => $itemDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"   => 1,
                            "VRPRECCOMVEN"   => bcdiv($comboItemPrice, "100", 2),  
                            "TXPRODCOMVEN"   => $itemDetails["TXPRODCOMVEN"],
                            "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                            "VRDESCCOMVEN"   => 0,
                            "NRLUGARMESA"    => "01",
                            "IDPRODIMPFIS"   => "N",
                            "CDLOJA"         => $generalParams["CDLOJA"],
                            "VRACRCOMVEN"    => 0,
                            "NRSEQPRODCOM"   => $productNrSeqProdCom,
                            "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                            "VRPRECCLCOMVEN" => 0,
                            "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                            "CDPRODPROMOCAO" => $CDPRODPROMO,
                            "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                            "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                            "DSOBSPEDDIGCMD" => null,
                            "ITEM_OBSERVATION" => []
                        );
                        
                        if (strlen($comboItem["customCode"]) === 5 && substr($comboItem["customCode"], 2, 1) === '|' && $comboItem['price']['value'] != "0.00"){

                            array_push($comboItens[0]['ITEM_OBSERVATION'], [
                                "CDFILIAL"        => $generalParams["CDFILIAL"],
                                "NRVENDAREST"     => $order["NRVENDAREST"],
                                "NRCOMANDA"       => $order["NRCOMANDA"],
                                "NRPRODCOMVEN"    => $comboDetails["NRPRODCOMVEN"],
                                "CDGRPOCOR"       => $itemDetails["CDGRPOCOR"],
                                "CDOCORR"         => $itemDetails["CDOCORR"],  
                                'NRPRODCOMVENOBS' => null,
                                'QTPRODCOMVENOBS' => $comboItem['amount']
                            ]);
                        }
                        
                        array_push($comboItens, $itemReturn);
                    }
                }
            }
        }
        return $comboItens;
    }
    
    private function processProductAsComboWithoutExtCode ($generalParams, $combo, $order, &$ITCOMANDAVEN_SEQ, &$comboItens = array(), $CDPRODPROMO = null) {
        $comboItens = array();

        for ($i = 0 ; $i < $combo["amount"] ; $i++) {
            foreach ($combo["properties"] as $comboItemAgrupador) {
                foreach ($comboItemAgrupador["options"] as $comboItem) {
                        if(strlen($comboItem['customCode']) > 5 && $comboItem['price']['value'] == 0){
                            throw new Exception\ParamsException("
                                    O subitem {$comboItem['name']}, do item {$combo['name']}, foi cadastrado sem preço e com o código externo no formato 'x.xx.xx.xxx.xx'.<br>
                                    Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                    Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'x.xx.xx.xxx.xx' ou no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                    O código externo deve ser igual ao cadastrado previamente no Retail.<br>
                                    Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
                        }
                        
                        $itemDetails = $this->factory->getProduct($comboItem["customCode"]);
                        
                        if (empty($itemDetails)){
                            
                            if(empty($comboItem['customCode'])){
                                throw new Exception\ParamsException("
                                    A observação ou acréscimo {$comboItem['name']} do produto {$combo['name']} não possui um código externo.<br>
                                    Verifique no Odhen Retail o código externo deste produto e faça a correção junto ao Delivery Direto.3<br>
                                    Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
                            }
                            throw new Exception\ParamsException("
                                Não existe uma observação ou acréscimo atrelado ao código externo informado.<br>
                                Observação ou acréscimo: {$comboItem['name']}.<br>
                                Código externo: {$comboItem['customCode']}.<br>
                                Produto pai: {$combo['name']}.<br>
                                Verifique no Odhen Retail o código externo deste produto e faça a correção junto ao Delivery Direto.4<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");
                        }
                        $itemDetails = array_merge($itemDetails, $comboItem);
                        
                        $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                        $ITCOMANDAVEN_SEQ++;
                    
                        $itemDetails["TXPRODCOMVEN"] = (isset($combo["comments"]) && !empty($combo["comments"])) ? (strtoupper($this->util->removeAcentos($combo["comments"]))) . "; " : "";
                        
                        $vrUnitPrice = $itemDetails["price"]["value"]; 
                        
                        if ($comboItem["amount"] > 1 && isset($itemDetails["price"]["value"])) {
                            $vrUnitPrice = $itemDetails["unitPrice"]["value"];
                        }
                        
                        $itemReturn = array(
                            "CDFILIAL"       => $generalParams["CDFILIAL"],
                            "NRVENDAREST"    => $order["NRVENDAREST"],
                            "NRCOMANDA"      => $order["NRCOMANDA"],
                            "NRPRODCOMVEN"   => $itemDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"      => $itemDetails["CDPRODUTO"],
                            "NMPRODUTO"      => $itemDetails["name"],
                            "QTPRODCOMVEN"   => $comboItem["amount"],
                            "VRPRECCOMVEN"   => bcdiv($vrUnitPrice, "100", 2),
                            "TXPRODCOMVEN"   => $itemDetails["TXPRODCOMVEN"],
                            "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                            "VRDESCCOMVEN"   => isset($itemDetails['VRDESCCOMVEN']) ? $itemDetails['VRDESCCOMVEN'] : 0,
                            "NRLUGARMESA"    => "01",
                            "IDPRODIMPFIS"   => "N",
                            "CDLOJA"         => $generalParams["CDLOJA"],
                            "VRACRCOMVEN"    => 0,
                            "NRSEQPRODCOM"   => null,
                            "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                            "VRPRECCLCOMVEN" => 0,
                            "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                            "CDPRODPROMOCAO" => null,
                            "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                            "NRINSCRCONS"    => $generalParams["CPF"],
                            "DSOBSPEDDIGCMD" => null
                        );
                    
                        array_push($comboItens, $itemReturn);
                }
            }
        }
        return $comboItens;
    }
    
    private function processObservationWithoutPrice($nrProdComVenKey, $observation, $amount, &$products) {
        foreach($products as &$product) {
            if($nrProdComVenKey != $product['NRPRODCOMVEN']) {
                continue;
            }

            for($i = 0; $i < $amount; $i++) {
                if(empty($product["TXPRODCOMVEN"])) {
                    $product["TXPRODCOMVEN"] = " $observation; ";
                } else {
                    $product["TXPRODCOMVEN"] .= "$observation; ";
                }
            }            
        }
        
        if(isset($product)) {
            unset($product);
        }
    }
    
    public function insertProduct ($product) {
        $this->factory->insertProduct($product);
    }

    public function insertObservation($observation) {
        $this->factory->insertObservation($observation);
    }

}
